@@ -1,6 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.0Beta
- *                    (C) Copyright 1997 R. Clint Whaley                     
+ *             Automatically Tuned Linear Algebra Software v3.8.3
+ *                    (C) Copyright 1997 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -10,23 +10,25 @@
  *   2. Redistributions in binary form must reproduce the above copyright
  *      notice, this list of conditions, and the following disclaimer in the
  *      documentation and/or other materials provided with the distribution.
- *   3. The name of the University, the ATLAS group, or the names of its 
- *      contributers may not be used to endorse or promote products derived
- *      from this software without specific written permission.
+ *   3. The name of the ATLAS group or the names of its contributers may
+ *      not be used to endorse or promote products derived from this
+ *      software without specific written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE. 
+ * POSSIBILITY OF SUCH DAMAGE.
  *
  */
+
+/* #define TEST_GPMM */
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -35,6 +37,9 @@
 #include "atlas_lvl3.h"
 #include "atlas_tst.h"
 #include "cblas.h"
+#ifdef TEST_GPMM
+   #include "atlas_pkblas.h"
+#endif
 
 #ifndef L2SIZE
    #define L2SIZE 4194304
@@ -87,22 +92,50 @@ void matgen(int M, int N, TYPE *A, int lda, int seed)
 #elif defined(TRUST_SMALL)
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,small_mm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TRUST_BIG)
+   #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,big_mm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TRUST_BPP)
+   #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,mmBPP)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TRUST_JIT)
+   #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,mmJITcp)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #else
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,f77gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #endif
 
-#if 1
+#ifdef TEST_GPMM
    #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
-         Mjoin(PATL,gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
-#else
+      Mjoin(PATL,gpmm)(PackGen, TA, PackGen, TB, PackGen, m, n, k, al, \
+                       A, 0, 0, lda, B, 0, 0, ldb, be, C, 0, 0, ldc)
+#elif defined(TEST_BIG)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,big_mm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_SMALLK)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,smallK_mm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_SMALL)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,small_mm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_BPP)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,mmBPP)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_JIT)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,mmJITcp)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_ALIASED)
    #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,aliased_gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#else
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #endif
 
-int mmcase(int TEST, char TA, char TB, int M, int N, int K, SCALAR alpha, 
-           TYPE *A, int lda, TYPE *B, int ldb, SCALAR beta, TYPE *C, int ldc,
-           TYPE *D, int ldd)
+int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
+	   SCALAR alpha, TYPE *A, int lda, TYPE *B, int ldb, SCALAR beta,
+	   TYPE *C, int ldc, TYPE *D, int ldd)
 {
    char *pc;
 #ifdef TREAL
@@ -119,12 +152,14 @@ int mmcase(int TEST, char TA, char TB, int M, int N, int K, SCALAR alpha,
    TYPE maxval, f1, ferr;
    static TYPE feps=0.0;
    static int itst=1;
-   int *L2, nL2=(1.3*L2SIZE)/sizeof(int);
+   /*int *L2, nL2=(1.3*L2SIZE)/sizeof(int);*/
    enum ATLAS_TRANS TAc, TBc;
+   double l2ret;
 
    if (!TEST) D = C;
-   if (nL2) L2 = malloc(nL2*sizeof(int));
-   if (TA == 'n' || TA == 'N') 
+   /*if (nL2) L2 = malloc(nL2*sizeof(int));*/
+   l2ret = ATL_flushcache( CACHESIZE );
+   if (TA == 'n' || TA == 'N')
    {
       matgen(M, K, A, lda, K*1112);
       TAc = AtlasNoTrans;
@@ -154,11 +189,15 @@ int mmcase(int TEST, char TA, char TB, int M, int N, int K, SCALAR alpha,
    printmat("C0", M, N, C, ldc);
 #endif
 
-   if (L2) /* invalidate L2 cache */
-   {
-      for (i=0; i != nL2; i++) L2[i] = 0.0; 
-      for (i=0; i != nL2; i++) j += L2[i];
-   }
+   /*
+     if (L2)
+     {
+     for (i=0; i != nL2; i++) L2[i] = 0.0;
+     for (i=0; i != nL2; i++) j += L2[i];
+     }*/
+
+   /* invalidate L2 cache */
+   l2ret = ATL_flushcache( -1 );
 
    t0 = time00();
    trusted_gemm(TAc, TBc, M, N, K, alpha, A, lda, B, ldb, beta, C, ldc);
@@ -179,18 +218,15 @@ int mmcase(int TEST, char TA, char TB, int M, int N, int K, SCALAR alpha,
 #ifndef TIMEONLY
    matgen(M, N, D, ldd, M*N);
 
-   if (L2)
-   {
-      for (i=0; i != nL2; i++) L2[i] = 0.0;  /* invalidate L2 cache */
-      for (i=0; i != nL2; i++) j += L2[i];  /* invalidate L2 cache */
-   }
+   /* invalidate L2 cache */
+   l2ret = ATL_flushcache( -1 );
 
    t0 = time00();
    test_gemm(TAc, TBc, M, N, K, alpha, A, lda, B, ldb, beta, D, ldd);
-   
+
    t2 = time00() - t0;
    if (t2 <= 0.0) t2 = mflop = 0.0;
-   else 
+   else
       #ifdef TCPLX
          mflop = ( ((8.0*M)*N)*K ) / (t2*1000000.0);
       #else
@@ -265,12 +301,13 @@ int mmcase(int TEST, char TA, char TB, int M, int N, int K, SCALAR alpha,
    itst++;
    PASSED = 1;
 #endif
-   free(L2);
+   /*free(L2);*/
+   l2ret = ATL_flushcache( 0 );
    return(PASSED);
 }
 
-int mmcase0(int MFLOP, char TA, char TB, int M, int N, int K, SCALAR alpha, 
-            int lda, int ldb, SCALAR beta, int ldc)
+int mmcase0(int MFLOP, int CACHESIZE, char TA, char TB, int M, int N, int K,
+	    SCALAR alpha, int lda, int ldb, SCALAR beta, int ldc)
 {
    char *pc;
 #ifdef TREAL
@@ -322,7 +359,7 @@ int mmcase0(int MFLOP, char TA, char TB, int M, int N, int K, SCALAR alpha,
    #endif
    nreps = MFLOP / mops;
    if (nreps < 1) nreps = 1;
-   if (TA == 'n' || TA == 'N') 
+   if (TA == 'n' || TA == 'N')
    {
       TAc = AtlasNoTrans;
       incA = lda * K;
@@ -347,7 +384,8 @@ int mmcase0(int MFLOP, char TA, char TB, int M, int N, int K, SCALAR alpha,
    incC = ldc*N;
    inc = incA + incB + incC;
    i = M*K + K*N + M*N;  /* amount of inc actually referenced */
-   nmat = ((L2SIZE/ATL_sizeof) + i)/i;
+   /* This is a hack; change to use of flushcache instead. */
+   nmat = ((CACHESIZE/ATL_sizeof) + i)/i;
    vp = malloc(ATL_MulBySize(nmat*inc)+ATL_Cachelen);
    ATL_assert(vp);
    C = c = ATL_AlignPtr(vp);
@@ -367,9 +405,9 @@ int mmcase0(int MFLOP, char TA, char TB, int M, int N, int K, SCALAR alpha,
    {
       trusted_gemm(TAc, TBc, M, N, K, alpha, a, lda, b, ldb, bet, c, ldc);
       c += inc; a += inc; b += inc;
-      if (c == st) 
-      { 
-         c = C; a = A; b = B; 
+      if (c == st)
+      {
+         c = C; a = A; b = B;
          if (bet == beta) bet = betinv;
          else bet = beta;
       }
@@ -391,14 +429,14 @@ int mmcase0(int MFLOP, char TA, char TB, int M, int N, int K, SCALAR alpha,
    {
       test_gemm(TAc, TBc, M, N, K, alpha, a, lda, b, ldb, bet, c, ldc);
       c += inc; a += inc; b += inc;
-      if (c == st) 
-      { 
-         c = C; a = A; b = B; 
+      if (c == st)
+      {
+         c = C; a = A; b = B;
          if (bet == beta) bet = betinv;
          else bet = beta;
       }
    }
-   
+
    t2 = time00() - t0;
    t2 /= nreps;
    if (t2 <= 0.0) t2 = mflop = 0.0;
@@ -415,18 +453,18 @@ int mmcase0(int MFLOP, char TA, char TB, int M, int N, int K, SCALAR alpha,
 
 void PrintUsage(char *nam)
 {
-   fprintf(stderr, "USAGE: %s -Side <nsides> L/R -Uplo <nuplo> L/U -Atrans <ntrans> n/t/c -Btrans <ntrans> n/t/c -Diag <ndiags> N/U -M <m1> <mN> <minc> -N <n1> <nN> <ninc> <k1> <kN> <kinc> -n <n> -m <m> -k <k> -a <nalphas> <alpha1> ... <alphaN> -b <nbetas> <beta1> ... <betaN> -Test <0/1> -F <mflops>\n", nam);
+   fprintf(stderr, "USAGE: %s -Side <nsides> L/R -Uplo <nuplo> L/U -Atrans <ntrans> n/t/c -Btrans <ntrans> n/t/c -Diag <ndiags> N/U -M <m1> <mN> <minc> -N <n1> <nN> <ninc> <k1> <kN> <kinc> -n <n> -m <m> -k <k> -a <nalphas> <alpha1> ... <alphaN> -b <nbetas> <beta1> ... <betaN> -Test <0/1> -F <mflops> -C <cachesize>\n", nam);
    exit(-1);
 }
 
 void GetFlags(int nargs, char *args[], int *TEST, int *nside,
-              enum ATLAS_SIDE **Side, int *nuplo, enum ATLAS_UPLO **Uplo, 
-              int *nta, enum ATLAS_TRANS **TransA, 
+              enum ATLAS_SIDE **Side, int *nuplo, enum ATLAS_UPLO **Uplo,
+              int *nta, enum ATLAS_TRANS **TransA,
               int *ntb, enum ATLAS_TRANS **TransB, int *ndiag,
-              enum ATLAS_DIAG **Diag, int *M0, int *MN, int *Minc, 
+              enum ATLAS_DIAG **Diag, int *M0, int *MN, int *Minc,
               int *N0, int *NN, int *Ninc, int *K0, int *KN, int *Kinc,
               int *nalphas, TYPE **alphas, int *nbetas, TYPE **betas,
-              int *LDA_IS_M, int *MFLOP)
+              int *LDA_IS_M, int *MFLOP, int *CACHESIZE)
 
 {
    char ch;
@@ -438,6 +476,11 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
    *M0 = *N0 = *K0 = -1;
    *nuplo = *nta = *ntb = *nside = *ndiag = *nalphas = *nbetas = -1;
    *MFLOP = *LDA_IS_M = 0;
+#ifdef L2SIZE
+   *CACHESIZE = L2SIZE;               /* Size of largest cache to flush */
+#else
+   *CACHESIZE = 4*1024*1024;
+#endif
 
    for (i=1; i < nargs; i++)
    {
@@ -446,6 +489,10 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
       case 'F':
          *MFLOP = atoi(args[++i]);
          break;
+      case 'C':
+            if( args[i+1] == NULL ) PrintUsage( args[0] );
+	    *CACHESIZE = 1024*atoi(args[++i]);
+            break;
       case 'S':
          *nside = atoi(args[++i]);
          *Side = malloc(*nside * sizeof(int));
@@ -632,10 +679,10 @@ main(int nargs, char *args[])
 /*
  *  tst <tst> <# TA> <TA's> <# TB's> <TB's> <M0> <MN> <incM> <N0> <NN> <incN>
  *      <K0> <KN> <incK> <# alphas> <alphas> <# betas> <betas>
- *          
+ *
  */
 {
-   int M0, MN, incM, N0, NN, incN, K0, KN, incK, lda, ldb, ldc, MFLOP; 
+   int M0, MN, incM, N0, NN, incN, K0, KN, incK, lda, ldb, ldc, MFLOP;
    int i, k, m, n, im, in, ik, ita, itb, ia, ib, nTA, nTB, nalph, nbeta;
    int itst=0, ipass=0, TEST, LDA_IS_M, MSAME=0, KSAME=0;
    int ndiag, nuplo, nside;
@@ -646,15 +693,16 @@ main(int nargs, char *args[])
       TYPE bet1[2] = {1.0, 0.0}, alp1[2] = {-1.0, 0.0};
    #endif
    char TA, TB;
-   enum ATLAS_SIDE *Side; 
+   enum ATLAS_SIDE *Side;
    enum ATLAS_UPLO *Uplo;
    enum ATLAS_TRANS *TransA, *TransB, TAc, TBc;
    enum ATLAS_DIAG *Diag;
+   int CACHESIZE;
 
-   GetFlags(nargs, args, &TEST, &nside, &Side, &nuplo, &Uplo, 
-            &nTA, &TransA, &nTB, &TransB, &ndiag, &Diag, 
+   GetFlags(nargs, args, &TEST, &nside, &Side, &nuplo, &Uplo,
+            &nTA, &TransA, &nTB, &TransB, &ndiag, &Diag,
             &M0, &MN, &incM, &N0, &NN, &incN, &K0, &KN, &incK,
-            &nalph, &alph, &nbeta, &beta, &LDA_IS_M, &MFLOP);
+            &nalph, &alph, &nbeta, &beta, &LDA_IS_M, &MFLOP,&CACHESIZE);
 
    if (M0 == -1)
    {
@@ -685,10 +733,10 @@ main(int nargs, char *args[])
  */
    if (MFLOP)
    {
-      mmcase0(10, 'n', 'n', 100, 100, 100, alp1, 100, 100, bet1, 100);
-      mmcase0(10, 'n', 't', 100, 100, 100, alp1, 100, 100, bet1, 100);
-      mmcase0(10, 't', 'n', 100, 100, 100, alp1, 100, 100, bet1, 100);
-      mmcase0(10, 't', 't', 100, 100, 100, alp1, 100, 100, bet1, 100);
+      mmcase0(10, 1, 'n', 'n', 100, 100, 100, alp1, 100, 100, bet1, 100);
+      mmcase0(10, 1, 'n', 't', 100, 100, 100, alp1, 100, 100, bet1, 100);
+      mmcase0(10, 1, 't', 'n', 100, 100, 100, alp1, 100, 100, bet1, 100);
+      mmcase0(10, 1, 't', 't', 100, 100, 100, alp1, 100, 100, bet1, 100);
    }
    else
    {
@@ -753,29 +801,31 @@ main(int nargs, char *args[])
                            else ldb = NN;
                            ldc = MN;
                         }
-                        
+
                         if (MFLOP)
                         {
                            ipass++;
-                           #ifdef TREAL
-                              mmcase0(MFLOP, TA, TB, m, n, k, alph[ia],
-                                      lda, ldb, beta[ib], ldc);
-                           #else
-                              mmcase0(MFLOP, TA, TB, m, n, k, alph+(ia SHIFT),
-                                      lda, ldb, beta+(ib SHIFT), ldc);
-                           #endif
+#ifdef TREAL
+                              mmcase0(MFLOP, CACHESIZE, TA, TB, m, n, k,
+				      alph[ia], lda, ldb, beta[ib], ldc);
+#else
+                              mmcase0(MFLOP, CACHESIZE, TA, TB, m, n, k,
+				      alph+(ia SHIFT), lda, ldb,
+				      beta+(ib SHIFT), ldc);
+#endif
                         }
                         else
                         {
-                           #ifdef TREAL
-                              ipass += mmcase(TEST, TA, TB, m, n, k, 
-                                              alph[ia], A, lda, B, ldb,
+#ifdef TREAL
+                              ipass += mmcase(TEST, CACHESIZE, TA, TB, m,
+					      n, k, alph[ia], A, lda, B, ldb,
                                               beta[ib], C, ldc, D,ldc);
-                           #else
-                              ipass += mmcase(TEST, TA, TB, m, n, k, 
-                                              alph+(ia SHIFT), A, lda, B, ldb,
-                                              beta+(ib SHIFT), C, ldc, D,ldc);
-                           #endif
+#else
+                              ipass += mmcase(TEST, CACHESIZE, TA, TB, m,
+					      n, k, alph+(ia SHIFT), A,
+					      lda, B, ldb, beta+(ib SHIFT),
+					      C, ldc, D,ldc);
+#endif
                         }
                      }
                   }
