@@ -1,6 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.0Beta
- *                    (C) Copyright 1999 R. Clint Whaley                     
+ *             Automatically Tuned Linear Algebra Software v3.8.3
+ *                    (C) Copyright 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -10,21 +10,21 @@
  *   2. Redistributions in binary form must reproduce the above copyright
  *      notice, this list of conditions, and the following disclaimer in the
  *      documentation and/or other materials provided with the distribution.
- *   3. The name of the University, the ATLAS group, or the names of its 
- *      contributers may not be used to endorse or promote products derived
- *      from this software without specific written permission.
+ *   3. The name of the ATLAS group or the names of its contributers may
+ *      not be used to endorse or promote products derived from this
+ *      software without specific written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE. 
+ * POSSIBILITY OF SUCH DAMAGE.
  *
  */
 #ifndef ATLAS_KERNEL3_H
@@ -825,53 +825,53 @@ void ATL_strputU_b0
    (const int N, const float *v, const float beta, float *A, const int lda);
 void ATL_strputL_b0
    (const int N, const float *v, const float beta, float *A, const int lda);
-void ATL_strsm0LLTN
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKLLTN
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0LLTU
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKLLTU
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0LLNN
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKLLNN
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0LLNU
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKLLNU
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0LUTN
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKLUTN
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0LUTU
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKLUTU
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0LUNN
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKLUNN
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0LUNU
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKLUNU
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0RLTN
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKRLTN
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0RLTU
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKRLTU
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0RLNN
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKRLNN
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0RLNU
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKRLNU
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0RUTN
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKRUTN
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0RUTU
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKRUTU
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0RUNN
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKRUNN
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
-void ATL_strsm0RUNU
-   (const int M, const int N, const float alpha, const float *A, 
+void ATL_strsmKRUNU
+   (const int M, const int N, const float alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_dsycopyU_a0
    (const int N, const double alpha, const double *A, const int lda, double *C);
@@ -961,58 +961,62 @@ void ATL_dtrputU_b0
    (const int N, const double *v, const double beta, double *A, const int lda);
 void ATL_dtrputL_b0
    (const int N, const double *v, const double beta, double *A, const int lda);
-void ATL_dtrsm0LLTN
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKLLTN
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0LLTU
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKLLTU
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0LLNN
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKLLNN
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0LLNU
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKLLNU
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0LUTN
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKLUTN
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0LUTU
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKLUTU
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0LUNN
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKLUNN
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0LUNU
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKLUNU
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0RLTN
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKRLTN
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0RLTU
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKRLTU
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0RLNN
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKRLNN
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0RLNU
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKRLNU
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0RUTN
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKRUTN
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0RUTU
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKRUTU
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0RUNN
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKRUNN
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
-void ATL_dtrsm0RUNU
-   (const int M, const int N, const double alpha, const double *A, 
+void ATL_dtrsmKRUNU
+   (const int M, const int N, const double alpha, const double *A,
     const int lda, double *C, const int ldc);
 
 /*
  * Complex level 3 kernel auxiliaries
  */
+void ATL_cCtrsmKL
+   (enum ATLAS_UPLO Uplo, enum ATLAS_TRANS Trans, enum ATLAS_DIAG Diag,
+    const int M, const int N, const float *alpha, const float *A,
+    const int lda, float *B, const int ldb);
 void ATL_checopy
    (const int N, const float *A, const int lda, float *C);
 void ATL_csycopy
@@ -1126,77 +1130,81 @@ void ATL_cheputU_bXi0
 void ATL_cheputL_bXi0
    (const int N, const float *v, const float *beta, float *A, const int lda);
 void ATL_ctrsm0LLTN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LLTU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LLNN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LLNU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LLCN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LLCU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LUTN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LUTU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LUNN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LUNU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LUCN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0LUCU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RLTN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RLTU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RLNN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RLNU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RLCN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RLCU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RUTN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RUTU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RUNN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RUNU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RUCN
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
 void ATL_ctrsm0RUCU
-   (const int M, const int N, const float *alpha, const float *A, 
+   (const int M, const int N, const float *alpha, const float *A,
     const int lda, float *C, const int ldc);
+void ATL_zCtrsmKL
+   (enum ATLAS_UPLO Uplo, enum ATLAS_TRANS Trans, enum ATLAS_DIAG Diag,
+    const int M, const int N, const double *alpha, const double *A,
+    const int lda, double *B, const int ldb);
 void ATL_zhecopy
    (const int N, const double *A, const int lda, double *C);
 void ATL_zsycopy
@@ -1310,76 +1318,76 @@ void ATL_zheputU_bXi0
 void ATL_zheputL_bXi0
    (const int N, const double *v, const double *beta, double *A, const int lda);
 void ATL_ztrsm0LLTN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LLTU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LLNN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LLNU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LLCN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LLCU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LUTN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LUTU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LUNN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LUNU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LUCN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0LUCU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RLTN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RLTU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RLNN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RLNU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RLCN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RLCU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RUTN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RUTU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RUNN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RUNU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RUCN
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 void ATL_ztrsm0RUCU
-   (const int M, const int N, const double *alpha, const double *A, 
+   (const int M, const int N, const double *alpha, const double *A,
     const int lda, double *C, const int ldc);
 
 #endif
