@@ -1,6 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.0Beta
- *                    (C) Copyright 1997 R. Clint Whaley                     
+ *             Automatically Tuned Linear Algebra Software v3.8.3
+ *                    (C) Copyright 1997 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -10,21 +10,21 @@
  *   2. Redistributions in binary form must reproduce the above copyright
  *      notice, this list of conditions, and the following disclaimer in the
  *      documentation and/or other materials provided with the distribution.
- *   3. The name of the University, the ATLAS group, or the names of its 
- *      contributers may not be used to endorse or promote products derived
- *      from this software without specific written permission.
+ *   3. The name of the ATLAS group or the names of its contributers may
+ *      not be used to endorse or promote products derived from this
+ *      software without specific written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE. 
+ * POSSIBILITY OF SUCH DAMAGE.
  *
  */
 
@@ -88,36 +88,36 @@
 #endif
 
 #ifndef CN_MNK_M
-   #define CN_MNK_M  TN_MNK_M  
-   #define CN_MNK_N  TN_MNK_N 
+   #define CN_MNK_M  TN_MNK_M
+   #define CN_MNK_N  TN_MNK_N
    #define CN_MNK_K  TN_MNK_K
    #define CN_MNK_MN TN_MNK_MN
    #define CN_MNK_GE TN_MNK_GE
 #endif
 #ifndef NC_MNK_M
-   #define NC_MNK_M  NT_MNK_M  
-   #define NC_MNK_N  NT_MNK_N 
+   #define NC_MNK_M  NT_MNK_M
+   #define NC_MNK_N  NT_MNK_N
    #define NC_MNK_K  NT_MNK_K
    #define NC_MNK_MN NT_MNK_MN
    #define NC_MNK_GE NT_MNK_GE
 #endif
 #ifndef CT_MNK_M
-   #define CT_MNK_M  TT_MNK_M  
-   #define CT_MNK_N  TT_MNK_N 
+   #define CT_MNK_M  TT_MNK_M
+   #define CT_MNK_N  TT_MNK_N
    #define CT_MNK_K  TT_MNK_K
    #define CT_MNK_MN TT_MNK_MN
    #define CT_MNK_GE TT_MNK_GE
 #endif
 #ifndef TC_MNK_M
-   #define TC_MNK_M  TT_MNK_M  
-   #define TC_MNK_N  TT_MNK_N 
+   #define TC_MNK_M  TT_MNK_M
+   #define TC_MNK_N  TT_MNK_N
    #define TC_MNK_K  TT_MNK_K
    #define TC_MNK_MN TT_MNK_MN
    #define TC_MNK_GE TT_MNK_GE
 #endif
 #ifndef CC_MNK_M
-   #define CC_MNK_M  TT_MNK_M  
-   #define CC_MNK_N  TT_MNK_N 
+   #define CC_MNK_M  TT_MNK_M
+   #define CC_MNK_N  TT_MNK_N
    #define CC_MNK_K  TT_MNK_K
    #define CC_MNK_MN TT_MNK_MN
    #define CC_MNK_GE TT_MNK_GE
@@ -126,15 +126,17 @@
 #define CPAT Mjoin(C_ATL_, PRE);
 
 #ifndef ATL_MaxMalloc
-   #define ATL_MaxMalloc 4194304
+   #define ATL_MaxMalloc 67108864
 #endif
 
 typedef void (*MAT2BLK)(int, int, const TYPE*, int, TYPE*, const SCALAR);
+typedef void (*MAT2BLK2)(const int, const int, const SCALAR, const TYPE*,
+                         const int, TYPE*, const int);
 typedef void (*MATSCAL)(const int, const int, const SCALAR, TYPE*, const int);
 typedef void (*PUTBLK)(int, int, TYPE*, TYPE*, int, const SCALAR);
 typedef void (*NBCLEANUP)(const TYPE*, const TYPE*, TYPE*, const int);
-typedef int (*MMINTR)(const enum ATLAS_TRANS, const enum ATLAS_TRANS, 
-                      const int, const int, const int, const SCALAR, 
+typedef int (*MMINTR)(const enum ATLAS_TRANS, const enum ATLAS_TRANS,
+                      const int, const int, const int, const SCALAR,
                       const TYPE *, const int, const TYPE *, const int,
                       const SCALAR, TYPE *, const int);
 typedef void (*NBMM0)(const int, const int, const int, const TYPE,
@@ -142,6 +144,8 @@ typedef void (*NBMM0)(const int, const int, const int, const TYPE,
                       const TYPE, TYPE*, const int);
 
 void ATL_xerbla(int p, char *rout, char *form, ...);
+int Mjoin(PATL,GetNB)(void);
+int Mjoin(PATL,GetNCNB)(void);
 
 void Mjoin(PATL, gescal_bX)(const int, const int, const SCALAR, TYPE*,
            const int);
@@ -150,54 +154,66 @@ void Mjoin(PATL, gescal_bn1)(const int, const int, const SCALAR, TYPE*,
 void Mjoin(PATL, gescal_b0)(const int, const int, const SCALAR, TYPE*,
            const int);
 
-void Mjoin(PATL,pNBmm_bX)(const int M, const int N, const int K, 
-                          const TYPE alpha, const TYPE *A, const int lda, 
-                          const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,pKBmm_bX)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
-void Mjoin(PATL,pMBmm_bX)(const int M, const int N, const int K, 
-                          const TYPE alpha, const TYPE *A, const int lda, 
-                          const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,pNBmm_bX)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
-void Mjoin(PATL,pNBmm_b1)(const int M, const int N, const int K, 
-                          const TYPE alpha, const TYPE *A, const int lda, 
-                          const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,pMBmm_bX)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
-void Mjoin(PATL,pMBmm_b1)(const int M, const int N, const int K, 
-                          const TYPE alpha, const TYPE *A, const int lda, 
-                          const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,pKBmm_b1)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
-void Mjoin(PATL,pNBmm_b0)(const int M, const int N, const int K, 
-                          const TYPE alpha, const TYPE *A, const int lda, 
-                          const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,pNBmm_b1)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
-void Mjoin(PATL,pMBmm_b0)(const int M, const int N, const int K, 
-                          const TYPE alpha, const TYPE *A, const int lda, 
-                          const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,pMBmm_b1)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
-void Mjoin(PATL,pKBmm)(const int M, const int N, const int K, 
-                       const TYPE alpha, const TYPE *A, const int lda, 
-                       const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,pKBmm_b0)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
+                          TYPE *C, const int ldc);
+void Mjoin(PATL,pNBmm_b0)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
+                          TYPE *C, const int ldc);
+void Mjoin(PATL,pMBmm_b0)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
+                          TYPE *C, const int ldc);
+void Mjoin(PATL,pKBmm)(const int M, const int N, const int K,
+                       const TYPE alpha, const TYPE *A, const int lda,
+                       const TYPE *B, const int ldb, const TYPE beta,
                        TYPE *C, const int ldc);
 
 void Mjoin(PATL,MBJBmm)(const int N, const int K, const TYPE *A, const TYPE *B,
                         const TYPE beta, TYPE *C, const int ldc);
-void Mjoin(PATL,IBJBmm)(int IB, int JB, int K, const TYPE *A, const TYPE *B, 
+void Mjoin(PATL,IBJBmm)(int IB, int JB, int K, const TYPE *A, const TYPE *B,
                         const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(PATL,IBNBmm)(const int M, const int K, const TYPE *A, const TYPE *B,
                         const TYPE beta, TYPE *C, const int ldc);
 #ifdef TCPLX
 
-void Mjoin(PATL,CNBmm_b0)(const int M, const int N, const int K, 
-                          const TYPE alpha, const TYPE *A, const int lda, 
-                          const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,CNBmm_b0)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
-void Mjoin(PATL,CNBmm_b1)(const int M, const int N, const int K, 
-                          const TYPE alpha, const TYPE *A, const int lda, 
-                          const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,CNBmm_b1)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
-void Mjoin(PATL,CNBmm_bX)(const int M, const int N, const int K, 
-                          const TYPE alpha, const TYPE *A, const int lda, 
-                          const TYPE *B, const int ldb, const TYPE beta, 
+void Mjoin(PATL,CNBmm_bX)(const int M, const int N, const int K,
+                          const TYPE alpha, const TYPE *A, const int lda,
+                          const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
 void Mjoin(PATL, gescal_bXi0)(const int, const int, const SCALAR, TYPE*,
            const int);
@@ -238,8 +254,8 @@ void Mjoin(PATL,col2blkConj2_aXi0)
 
 void Mjoin(PATL,mmJIK2)
    (int K, int nMb, int nNb, int nKb, int ib, int jb, int kb,
-    const SCALAR alpha, const TYPE *pA0, const TYPE *B, int ldb, TYPE *pB0, 
-    int incB, MAT2BLK B2blk, const SCALAR beta, TYPE *C, int ldc, 
+    const SCALAR alpha, const TYPE *pA0, const TYPE *B, int ldb, TYPE *pB0,
+    int incB, MAT2BLK B2blk, const SCALAR beta, TYPE *C, int ldc,
     MATSCAL gescal, NBMM0 NBmm0);
 
 void Mjoin(PATL,mmIJK2)
@@ -254,70 +270,78 @@ void Mjoin(PATL,putblk_bX)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR
 void Mjoin(PATL,putblk_bn1)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
 void Mjoin(PATL,putblk_b1)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
 void Mjoin(PATL,putblk_b0)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
+void ATL_gereal2cplx(const int M, const int N, TYPE *alpha, TYPE *R, int ldr,
+                     TYPE *I, int ldi, TYPE *beta, TYPE *C, int ldc);
 
-void NBmm_b1(const int M, const int N, const int K, const TYPE alpha, 
+void NBmm_b1(const int M, const int N, const int K, const TYPE alpha,
              const TYPE *A, const int lda, const TYPE *B, const int ldb,
              const TYPE beta, TYPE *C, const int ldc);
-void NBmm_b0(const int M, const int N, const int K, const TYPE alpha, 
+void NBmm_b0(const int M, const int N, const int K, const TYPE alpha,
              const TYPE *A, const int lda, const TYPE *B, const int ldb,
              const TYPE beta, TYPE *C, const int ldc);
-void NBmm_bX(const int M, const int N, const int K, const TYPE alpha, 
+void NBmm_bX(const int M, const int N, const int K, const TYPE alpha,
              const TYPE *A, const int lda, const TYPE *B, const int ldb,
              const TYPE beta, TYPE *C, const int ldc);
-void Mjoin(PATL,mm_Bout2)(int K, int nMb, int nNb, int nKb, int ib, int jb, 
-                          int kb, const SCALAR alpha, const TYPE *pA0, 
-                          const TYPE *B, int ldb, TYPE *pB0, int incB, 
+void Mjoin(PATL,mmJIK2)(int K, int nMb, int nNb, int nKb, int ib, int jb,
+                          int kb, const SCALAR alpha, const TYPE *pA0,
+                          const TYPE *B, int ldb, TYPE *pB0, int incB,
                           MAT2BLK B2blk, const SCALAR beta, TYPE *C, int ldc,
                           TYPE *pC, PUTBLK putblk, NBMM0 NBmm0);
 
-void Mjoin(PATL,mm_Aout2)(int K, int nMb, int nNb, int nKb, int ib, int jb,
+void Mjoin(PATL,mmIJK2)(int K, int nMb, int nNb, int nKb, int ib, int jb,
                           int kb, const SCALAR alpha, const TYPE *A, int lda,
                           TYPE *pA0, int incA, MAT2BLK A2blk, const TYPE *pB0,
-                          const SCALAR beta, TYPE *C, int ldc, TYPE *pC, 
+                          const SCALAR beta, TYPE *C, int ldc, TYPE *pC,
                           PUTBLK putblk, NBMM0 NBmm0);
 
 
 void Mjoin(PATL,aliased_gemm)
-   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB, 
-    const int M, const int N, int K, const SCALAR alpha, 
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+    const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,f77gemm)
-   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB, 
-    const int M, const int N, int K, const SCALAR alpha, 
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+    const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,gemm)
-   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB, 
-    const int M, const int N, int K, const SCALAR alpha, 
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+    const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,small_mm)
-   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB, 
-    const int M, const int N, int K, const SCALAR alpha, 
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+    const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,big_mm)
-   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB, 
-    const int M, const int N, int K, const SCALAR alpha, 
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+   (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+    const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 #endif
 
-int  Mjoin(PATL,NCmmJIK)(const enum ATLAS_TRANS, const enum ATLAS_TRANS, 
-                         const int, const int, const int, const SCALAR, 
+#ifdef USERGEMM
+int  Mjoin(PATU,usergemm)(const enum ATLAS_TRANS, const enum ATLAS_TRANS,
+                          const int, const int, const int, const SCALAR,
+                          const TYPE*, const int, const TYPE*,
+                          const int, const SCALAR, TYPE*, const int);
+#endif
+int  Mjoin(PATL,NCmmJIK)(const enum ATLAS_TRANS, const enum ATLAS_TRANS,
+                         const int, const int, const int, const SCALAR,
                          const TYPE*, const int, const TYPE*,
                          const int, const SCALAR, TYPE*, const int);
-int  Mjoin(PATL,NCmmIJK)(const enum ATLAS_TRANS, const enum ATLAS_TRANS, 
-                         const int, const int, const int, const SCALAR, 
+int  Mjoin(PATL,NCmmIJK)(const enum ATLAS_TRANS, const enum ATLAS_TRANS,
+                         const int, const int, const int, const SCALAR,
                          const TYPE*, const int, const TYPE*,
                          const int, const SCALAR, TYPE*, const int);
-int  Mjoin(PATL,NCmmJIK_c)(const enum ATLAS_TRANS, const enum ATLAS_TRANS, 
-                           const int, const int, const int, const SCALAR, 
+int  Mjoin(PATL,NCmmJIK_c)(const enum ATLAS_TRANS, const enum ATLAS_TRANS,
+                           const int, const int, const int, const SCALAR,
                            const TYPE*, const int, const TYPE*,
                            const int, const SCALAR, TYPE*, const int);
-int  Mjoin(PATL,NCmmIJK_c)(const enum ATLAS_TRANS, const enum ATLAS_TRANS, 
-                           const int, const int, const int, const SCALAR, 
+int  Mjoin(PATL,NCmmIJK_c)(const enum ATLAS_TRANS, const enum ATLAS_TRANS,
+                           const int, const int, const int, const SCALAR,
                            const TYPE*, const int, const TYPE*,
                            const int, const SCALAR, TYPE*, const int);
 
@@ -335,130 +359,154 @@ void Mjoin(PATL,row2blkT_a1)(int, int, const TYPE*, int, TYPE*, const SCALAR);
 void Mjoin(PATL,col2blk2_a1)(int, int, const TYPE*, int, TYPE*, const SCALAR);
 void Mjoin(PATL,col2blk_a1)(int, int, const TYPE*, int, TYPE*, const SCALAR);
 
-int Mjoin(PATL,mmJIK)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB, 
-                      const int M, const int N, const int K, 
+int Mjoin(PATL,mmJITcp)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+                        const int M, const int N, const int K,
+                        const SCALAR alpha, const TYPE *A, const int lda,
+                        const TYPE *B, const int ldb, const SCALAR beta,
+                        TYPE *C, const int ldc);
+int Mjoin(PATL,mmJIK)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+                      const int M, const int N, const int K,
                       const SCALAR alpha, const TYPE *A, const int lda,
                       const TYPE *B, const int ldb, const SCALAR beta,
                       TYPE *C, const int ldc);
-int Mjoin(PATL,mmIJK)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB, 
-                      const int M, const int N, const int K, 
+int Mjoin(PATL,mmIJK)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+                      const int M, const int N, const int K,
+                      const SCALAR alpha, const TYPE *A, const int lda,
+                      const TYPE *B, const int ldb, const SCALAR beta,
+                      TYPE *C, const int ldc);
+int Mjoin(PATL,mmJKI)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+                      const int M, const int N, const int K,
                       const SCALAR alpha, const TYPE *A, const int lda,
                       const TYPE *B, const int ldb, const SCALAR beta,
                       TYPE *C, const int ldc);
 
+void Mjoin(PATL,mmK)
+   (int M, int m, int N, int n, int nblk, int kr, int KR, const SCALAR alphaA,
+    const SCALAR alphaB, const SCALAR beta, const TYPE *A, const int lda,
+    const int incA, TYPE *pA, const int incAW, const TYPE *B, const int ldb,
+    const int incB, TYPE *pB, const int incBW, TYPE *C, const int ldc,
+    MAT2BLK2 A2blk, MAT2BLK2 B2blk, NBMM0 NBmm0, NBMM0 NBmm1);
+
+int Mjoin(PATL,mmBPP)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+                      const int M, const int N, const int K,
+                      const SCALAR alpha, const TYPE *A, const int lda,
+                      const TYPE *B, const int ldb, const SCALAR beta,
+                      TYPE *C, const int ldc);
+
+
 void Mjoin(PATL,gemmTT)
    (const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,aliased_gemmTT)
    (const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,gemmTN)
    (const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,aliased_gemmTN)
    (const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,gemmNT)
    (const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,aliased_gemmNT)
    (const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,gemmNN)
    (const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void Mjoin(PATL,aliased_gemmNN)
    (const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 
 
 void NCmmNNIJK_c
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmNTIJK_c
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmTNIJK_c
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmTTIJK_c
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmNNIJK
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmNTIJK
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmTNIJK
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmTTIJK
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmNNJIK_c
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmNTJIK_c
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmTNJIK_c
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmTTJIK_c
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmNNJIK
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmNTJIK
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmTNJIK
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
 void NCmmTTJIK
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha, 
+    const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
- 
+
 #endif
