@@ -1,6 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.0Beta
- *                    (C) Copyright 1999 R. Clint Whaley                     
+ *             Automatically Tuned Linear Algebra Software v3.8.3
+ *                    (C) Copyright 1999 R. Clint Whaley
  *
  * Code contributers : R. Clint Whaley, Antoine P. Petitet
  *
@@ -12,49 +12,67 @@
  *   2. Redistributions in binary form must reproduce the above copyright
  *      notice, this list of conditions, and the following disclaimer in the
  *      documentation and/or other materials provided with the distribution.
- *   3. The name of the University, the ATLAS group, or the names of its 
- *      contributers may not be used to endorse or promote products derived
- *      from this software without specific written permission.
+ *   3. The name of the ATLAS group or the names of its contributers may
+ *      not be used to endorse or promote products derived from this
+ *      software without specific written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE. 
+ * POSSIBILITY OF SUCH DAMAGE.
  *
  */
 
 #ifndef ATLAS_TST_H
    #define ATLAS_TST_H
 
-double time00();
+#include "atlas_enum.h"
 
+double time00();
+#ifndef UseCRand
+   void ATL_srand(int iseed);
+   int ATL_rand(void);
+   #define dumb_seed(iseed_) ATL_srand(iseed_)
+   #define dumb_rand() ( 0.5 - ((double)ATL_rand())/(2147483648.0) )
+#else
+   #define dumb_seed(iseed_) srand(iseed_)
+   #ifndef RAND_MAX  /* rather dangerous non-ansi workaround */
+      #define RAND_MAX ((unsigned long)(1<<30))
+   #endif
+   #define dumb_rand() ( 0.5 - ((double)rand())/((double)RAND_MAX) )
+#endif
+
+void ATL_ststsqtran(const int N, float *A, const int lda);
 void ATL_sgeprint
    (char *mat, const int M, const int N, const float *A, const int lda);
 
 float ATL_sgediffnrm1
-   (const int M, const int N, const float *A, const int lda, 
+   (const int M, const int N, const float *A, const int lda,
     const float *B, const int ldb);
+float ATL_shediffnrm
+   (const enum ATLAS_ORDER Order, const enum ATLAS_UPLO Uplo, const int N,
+    const float *A0, const int ld0, const float *A1, const int ld1);
 float ATL_sinfnrm(const int N, const float *X, const int incX);
 float ATL_sgenrm1
    (const int M, const int N, const float *A, const int lda);
 float ATL_strnrm1
-   (const enum ATLAS_UPLO Upper, const enum ATLAS_DIAG Diag, const int N, 
+   (const enum ATLAS_UPLO Upper, const enum ATLAS_DIAG Diag, const int N,
     const float *A, const int lda);
 float ATL_sgbnrm1
-   (const int M, const int N, const int KL, const int KU, 
+   (const int M, const int N, const int KL, const int KU,
     const float *A, const int lda);
 float ATL_stpnrm1
    (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG, const int N,
     const float *A);
 float ATL_stbnrm1
-   (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG, 
+   (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG,
     const int N, const int K, const float *A, const int LDA);
 float ATL_ssynrm
    (const enum ATLAS_UPLO UPLO, const int N, const float *A, const int LDA);
@@ -65,43 +83,47 @@ float ATL_sspnrm
 float ATL_shpnrm
    (const enum ATLAS_UPLO UPLO, const int N, const float *A);
 float ATL_ssbnrm
-   (const enum ATLAS_UPLO UPLO, const int N, const int K, 
+   (const enum ATLAS_UPLO UPLO, const int N, const int K,
     const float *A, const int LDA);
 float ATL_shbnrm
-   (const enum ATLAS_UPLO UPLO, const int N, const int K, 
+   (const enum ATLAS_UPLO UPLO, const int N, const int K,
     const float *A, const int LDA);
 
 void ATL_sgefillgap(const int M, const int N, float *A, const int lda0);
 int ATL_sgechkgap(const int M0, const int N, float *A, const int lda0);
-void ATL_strgen(const enum ATLAS_UPLO Uplo, const enum ATLAS_DIAG Diag, 
+void ATL_strgen(const enum ATLAS_UPLO Uplo, const enum ATLAS_DIAG Diag,
                 const int N, float *A, const int lda, const int seed);
 void ATL_sgegen(const int M0, const int N, float *A, const int lda,
                 const int seed);
 float ATL_sepsilon(void);
-void ATL_svdiff(const int N, const float *X, const int incX, 
+void ATL_svdiff(const int N, const float *X, const int incX,
                 const float *Y, const int incY, float *Z, const int incZ);
 void ATL_sgediff(const int M, const int N, const float *A, const int lda,
                  const float *B, const int ldb, float *C, const int ldc);
+void ATL_dtstsqtran(const int N, double *A, const int lda);
 void ATL_dgeprint
    (char *mat, const int M, const int N, const double *A, const int lda);
 
 double ATL_dgediffnrm1
-   (const int M, const int N, const double *A, const int lda, 
+   (const int M, const int N, const double *A, const int lda,
     const double *B, const int ldb);
+double ATL_dhediffnrm
+   (const enum ATLAS_ORDER Order, const enum ATLAS_UPLO Uplo, const int N,
+    const double *A0, const int ld0, const double *A1, const int ld1);
 double ATL_dinfnrm(const int N, const double *X, const int incX);
 double ATL_dgenrm1
    (const int M, const int N, const double *A, const int lda);
 double ATL_dtrnrm1
-   (const enum ATLAS_UPLO Upper, const enum ATLAS_DIAG Diag, const int N, 
+   (const enum ATLAS_UPLO Upper, const enum ATLAS_DIAG Diag, const int N,
     const double *A, const int lda);
 double ATL_dgbnrm1
-   (const int M, const int N, const int KL, const int KU, 
+   (const int M, const int N, const int KL, const int KU,
     const double *A, const int lda);
 double ATL_dtpnrm1
    (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG, const int N,
     const double *A);
 double ATL_dtbnrm1
-   (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG, 
+   (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG,
     const int N, const int K, const double *A, const int LDA);
 double ATL_dsynrm
    (const enum ATLAS_UPLO UPLO, const int N, const double *A, const int LDA);
@@ -112,43 +134,47 @@ double ATL_dspnrm
 double ATL_dhpnrm
    (const enum ATLAS_UPLO UPLO, const int N, const double *A);
 double ATL_dsbnrm
-   (const enum ATLAS_UPLO UPLO, const int N, const int K, 
+   (const enum ATLAS_UPLO UPLO, const int N, const int K,
     const double *A, const int LDA);
 double ATL_dhbnrm
-   (const enum ATLAS_UPLO UPLO, const int N, const int K, 
+   (const enum ATLAS_UPLO UPLO, const int N, const int K,
     const double *A, const int LDA);
 
 void ATL_dgefillgap(const int M, const int N, double *A, const int lda0);
 int ATL_dgechkgap(const int M0, const int N, double *A, const int lda0);
-void ATL_dtrgen(const enum ATLAS_UPLO Uplo, const enum ATLAS_DIAG Diag, 
+void ATL_dtrgen(const enum ATLAS_UPLO Uplo, const enum ATLAS_DIAG Diag,
                 const int N, double *A, const int lda, const int seed);
 void ATL_dgegen(const int M0, const int N, double *A, const int lda,
                 const int seed);
 double ATL_depsilon(void);
-void ATL_dvdiff(const int N, const double *X, const int incX, 
+void ATL_dvdiff(const int N, const double *X, const int incX,
                 const double *Y, const int incY, double *Z, const int incZ);
 void ATL_dgediff(const int M, const int N, const double *A, const int lda,
                  const double *B, const int ldb, double *C, const int ldc);
+void ATL_ctstsqtran(const int N, float *A, const int lda);
 void ATL_cgeprint
    (char *mat, const int M, const int N, const float *A, const int lda);
 
 float ATL_cgediffnrm1
-   (const int M, const int N, const float *A, const int lda, 
+   (const int M, const int N, const float *A, const int lda,
     const float *B, const int ldb);
+float ATL_chediffnrm
+   (const enum ATLAS_ORDER Order, const enum ATLAS_UPLO Uplo, const int N,
+    const float *A0, const int ld0, const float *A1, const int ld1);
 float ATL_cinfnrm(const int N, const float *X, const int incX);
 float ATL_cgenrm1
    (const int M, const int N, const float *A, const int lda);
 float ATL_ctrnrm1
-   (const enum ATLAS_UPLO Upper, const enum ATLAS_DIAG Diag, const int N, 
+   (const enum ATLAS_UPLO Upper, const enum ATLAS_DIAG Diag, const int N,
     const float *A, const int lda);
 float ATL_cgbnrm1
-   (const int M, const int N, const int KL, const int KU, 
+   (const int M, const int N, const int KL, const int KU,
     const float *A, const int lda);
 float ATL_ctpnrm1
    (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG, const int N,
     const float *A);
 float ATL_ctbnrm1
-   (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG, 
+   (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG,
     const int N, const int K, const float *A, const int LDA);
 float ATL_csynrm
    (const enum ATLAS_UPLO UPLO, const int N, const float *A, const int LDA);
@@ -159,43 +185,47 @@ float ATL_cspnrm
 float ATL_chpnrm
    (const enum ATLAS_UPLO UPLO, const int N, const float *A);
 float ATL_csbnrm
-   (const enum ATLAS_UPLO UPLO, const int N, const int K, 
+   (const enum ATLAS_UPLO UPLO, const int N, const int K,
     const float *A, const int LDA);
 float ATL_chbnrm
-   (const enum ATLAS_UPLO UPLO, const int N, const int K, 
+   (const enum ATLAS_UPLO UPLO, const int N, const int K,
     const float *A, const int LDA);
 
 void ATL_cgefillgap(const int M, const int N, float *A, const int lda0);
 int ATL_cgechkgap(const int M0, const int N, float *A, const int lda0);
-void ATL_ctrgen(const enum ATLAS_UPLO Uplo, const enum ATLAS_DIAG Diag, 
+void ATL_ctrgen(const enum ATLAS_UPLO Uplo, const enum ATLAS_DIAG Diag,
                 const int N, float *A, const int lda, const int seed);
 void ATL_cgegen(const int M0, const int N, float *A, const int lda,
                 const int seed);
 float ATL_cepsilon(void);
-void ATL_cvdiff(const int N, const float *X, const int incX, 
+void ATL_cvdiff(const int N, const float *X, const int incX,
                 const float *Y, const int incY, float *Z, const int incZ);
 void ATL_cgediff(const int M, const int N, const float *A, const int lda,
                  const float *B, const int ldb, float *C, const int ldc);
+void ATL_ztstsqtran(const int N, double *A, const int lda);
 void ATL_zgeprint
    (char *mat, const int M, const int N, const double *A, const int lda);
 
 double ATL_zgediffnrm1
-   (const int M, const int N, const double *A, const int lda, 
+   (const int M, const int N, const double *A, const int lda,
     const double *B, const int ldb);
+double ATL_zhediffnrm
+   (const enum ATLAS_ORDER Order, const enum ATLAS_UPLO Uplo, const int N,
+    const double *A0, const int ld0, const double *A1, const int ld1);
 double ATL_zinfnrm(const int N, const double *X, const int incX);
 double ATL_zgenrm1
    (const int M, const int N, const double *A, const int lda);
 double ATL_ztrnrm1
-   (const enum ATLAS_UPLO Upper, const enum ATLAS_DIAG Diag, const int N, 
+   (const enum ATLAS_UPLO Upper, const enum ATLAS_DIAG Diag, const int N,
     const double *A, const int lda);
 double ATL_zgbnrm1
-   (const int M, const int N, const int KL, const int KU, 
+   (const int M, const int N, const int KL, const int KU,
     const double *A, const int lda);
 double ATL_ztpnrm1
    (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG, const int N,
     const double *A);
 double ATL_ztbnrm1
-   (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG, 
+   (const enum ATLAS_UPLO UPLO, const enum ATLAS_DIAG DIAG,
     const int N, const int K, const double *A, const int LDA);
 double ATL_zsynrm
    (const enum ATLAS_UPLO UPLO, const int N, const double *A, const int LDA);
@@ -206,24 +236,75 @@ double ATL_zspnrm
 double ATL_zhpnrm
    (const enum ATLAS_UPLO UPLO, const int N, const double *A);
 double ATL_zsbnrm
-   (const enum ATLAS_UPLO UPLO, const int N, const int K, 
+   (const enum ATLAS_UPLO UPLO, const int N, const int K,
     const double *A, const int LDA);
 double ATL_zhbnrm
-   (const enum ATLAS_UPLO UPLO, const int N, const int K, 
+   (const enum ATLAS_UPLO UPLO, const int N, const int K,
     const double *A, const int LDA);
 
 void ATL_zgefillgap(const int M, const int N, double *A, const int lda0);
 int ATL_zgechkgap(const int M0, const int N, double *A, const int lda0);
-void ATL_ztrgen(const enum ATLAS_UPLO Uplo, const enum ATLAS_DIAG Diag, 
+void ATL_ztrgen(const enum ATLAS_UPLO Uplo, const enum ATLAS_DIAG Diag,
                 const int N, double *A, const int lda, const int seed);
 void ATL_zgegen(const int M0, const int N, double *A, const int lda,
                 const int seed);
 double ATL_zepsilon(void);
-void ATL_zvdiff(const int N, const double *X, const int incX, 
+void ATL_zvdiff(const int N, const double *X, const int incX,
                 const double *Y, const int incY, double *Z, const int incZ);
 void ATL_zgediff(const int M, const int N, const double *A, const int lda,
                  const double *B, const int ldb, double *C, const int ldc);
 
+/*
+ * Wrappers so that C can call F77 LAPACK
+ */
+int ATL_sf77getri
+   (const enum ATLAS_ORDER, const int, float*, const int, int*,
+    float*, int*);
+int ATL_sf77getrf
+   (const enum ATLAS_ORDER, const int, const int, float*, const int, int*);
+int ATL_sf77potrf(const enum ATLAS_UPLO, const int, float*, const int);
+int ATL_sf77lauum(const enum ATLAS_UPLO, const int, float*, const int);
+int ATL_sf77trtri(const enum ATLAS_UPLO, const enum ATLAS_DIAG, const int,
+                  float*, const int);
+int ATL_sf77posv(const enum ATLAS_UPLO, const int, const int, float*, const int, float*, const int);
+int ATL_sf77gesv(const int, const int, float*, const int, int*, float*, const int);
+int ATL_sf77gels(const enum ATLAS_TRANS, const int, const int, const int, float*, const int, float*, const int);
+int ATL_df77getri
+   (const enum ATLAS_ORDER, const int, double*, const int, int*,
+    double*, int*);
+int ATL_df77getrf
+   (const enum ATLAS_ORDER, const int, const int, double*, const int, int*);
+int ATL_df77potrf(const enum ATLAS_UPLO, const int, double*, const int);
+int ATL_df77lauum(const enum ATLAS_UPLO, const int, double*, const int);
+int ATL_df77trtri(const enum ATLAS_UPLO, const enum ATLAS_DIAG, const int,
+                  double*, const int);
+int ATL_df77posv(const enum ATLAS_UPLO, const int, const int, double*, const int, double*, const int);
+int ATL_df77gesv(const int, const int, double*, const int, int*, double*, const int);
+int ATL_df77gels(const enum ATLAS_TRANS, const int, const int, const int, double*, const int, double*, const int);
+int ATL_cf77getri
+   (const enum ATLAS_ORDER, const int, float*, const int, int*,
+    float*, int*);
+int ATL_cf77getrf
+   (const enum ATLAS_ORDER, const int, const int, float*, const int, int*);
+int ATL_cf77potrf(const enum ATLAS_UPLO, const int, float*, const int);
+int ATL_cf77lauum(const enum ATLAS_UPLO, const int, float*, const int);
+int ATL_cf77trtri(const enum ATLAS_UPLO, const enum ATLAS_DIAG, const int,
+                  float*, const int);
+int ATL_cf77posv(const enum ATLAS_UPLO, const int, const int, float*, const int, float*, const int);
+int ATL_cf77gesv(const int, const int, float*, const int, int*, float*, const int);
+int ATL_cf77gels(const enum ATLAS_TRANS, const int, const int, const int, float*, const int, float*, const int);
+int ATL_zf77getri
+   (const enum ATLAS_ORDER, const int, double*, const int, int*,
+    double*, int*);
+int ATL_zf77getrf
+   (const enum ATLAS_ORDER, const int, const int, double*, const int, int*);
+int ATL_zf77potrf(const enum ATLAS_UPLO, const int, double*, const int);
+int ATL_zf77lauum(const enum ATLAS_UPLO, const int, double*, const int);
+int ATL_zf77trtri(const enum ATLAS_UPLO, const enum ATLAS_DIAG, const int,
+                  double*, const int);
+int ATL_zf77posv(const enum ATLAS_UPLO, const int, const int, double*, const int, double*, const int);
+int ATL_zf77gesv(const int, const int, double*, const int, int*, double*, const int);
+int ATL_zf77gels(const enum ATLAS_TRANS, const int, const int, const int, double*, const int, double*, const int);
 /*
  * =====================================================================
  * Prototypes for C-callable F77 interface to the Level 1 BLAS routines
@@ -237,7 +318,7 @@ void       ATL_cf77rotg
 ( float  *,        const float  *,  float  *,        float  * );
 void       ATL_zf77rotg
 ( double *,        const double *,  double *,        double * );
- 
+
 void       ATL_sf77rotmg
 ( float  *,        float  *,        float  *,        const float,
   float  * );
@@ -262,7 +343,7 @@ float      ATL_scf77asum
 ( const int,       const float  *,  const int );
 double     ATL_dzf77asum
 ( const int,       const double *,  const int );
- 
+
 int        ATL_isf77amax
 ( const int,       const float  *,  const int );
 int        ATL_idf77amax
@@ -271,7 +352,7 @@ int        ATL_icf77amax
 ( const int,       const float  *,  const int );
 int        ATL_izf77amax
 ( const int,       const double *,  const int );
- 
+
 void       ATL_sf77scal
 ( const int,       const float,     float  *,        const int );
 void       ATL_df77scal
@@ -284,7 +365,24 @@ void       ATL_csf77scal
 ( const int,       const float,     float  *,        const int );
 void       ATL_zdf77scal
 ( const int,       const double,    double *,        const int );
-                                                                                
+
+void ATL_sf77set(const int, const float, float*, const int);
+void ATL_df77set(const int, const double, double*, const int);
+void ATL_cf77set(const int, const float*, float*, const int);
+void ATL_zf77set(const int, const double*, double*, const int);
+void ATL_sf77axpby
+   (const int, const float, const float*, const int, const float,
+    float*, const int);
+void ATL_df77axpby
+   (const int, const double, const double*, const int, const double,
+    double*, const int);
+void ATL_cf77axpby
+   (const int, const float*, const float*, const int, const float*,
+    float*, const int);
+void ATL_zf77axpby
+   (const int, const double*, const double*, const int, const double*,
+    double*, const int);
+
 void       ATL_sf77axpy
 ( const int,       const float,     const float  *,  const int,
   float  *,        const int );
@@ -297,10 +395,10 @@ void       ATL_cf77axpy
 void       ATL_zf77axpy
 ( const int,       const double *,  const double *,  const int,
   double *,        const int );
- 
+
 void       ATL_sf77copy
 ( const int,       const float  *,  const int,       float  *,
-  const int );  
+  const int );
 void       ATL_df77copy
 ( const int,       const double *,  const int,       double *,
   const int );
@@ -323,7 +421,7 @@ void       ATL_cf77swap
 void       ATL_zf77swap
 ( const int,       double *,        const int,       double *,
   const int );
- 
+
 void       ATL_sf77rot
 ( const int,       float  *,        const int,       float  *,
   const int,       const float,     const float  );
@@ -343,7 +441,7 @@ void       ATL_sf77rotm
 void       ATL_df77rotm
 ( const int,       double *,        const int,       double *,
   const int,       const double * );
- 
+
 float      ATL_sf77dot
 ( const int,       const float  *,  const int,       const float  *,
   const int );
@@ -362,7 +460,7 @@ void       ATL_zf77dotu_sub
 void       ATL_zf77dotc_sub
 ( const int,       const double *,  const int,       const double *,
   const int,       double * );
- 
+
 float      ATL_sdsf77dot
 ( const int,       const float,     const float  *,  const int,
   const float  *,  const int );
