@@ -1,6 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.0Beta
- *                    (C) Copyright 1999 R. Clint Whaley                     
+ *             Automatically Tuned Linear Algebra Software v3.8.3
+ *                    (C) Copyright 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -10,21 +10,21 @@
  *   2. Redistributions in binary form must reproduce the above copyright
  *      notice, this list of conditions, and the following disclaimer in the
  *      documentation and/or other materials provided with the distribution.
- *   3. The name of the University, the ATLAS group, or the names of its 
- *      contributers may not be used to endorse or promote products derived
- *      from this software without specific written permission.
+ *   3. The name of the ATLAS group or the names of its contributers may
+ *      not be used to endorse or promote products derived from this
+ *      software without specific written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE. 
+ * POSSIBILITY OF SUCH DAMAGE.
  *
  */
 
@@ -48,135 +48,135 @@
 #define NCmm0 Mjoin(Mjoin(PATL,JIK),NBnam)
 #define NCmm00 Mjoin(PATL,JIK)
 void Mjoin(Mjoin(Mjoin(NCmm0,NN),0x0x0),_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NN),0x0x0),_a1_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NN),0x0x0),_aX_b1)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NN),0x0x0),_a1_b1)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NN),0x0x0),_aX_b0)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NN),0x0x0),_a1_b0)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm00,Mjoin(0x0x,KB)),NN),0x0x0_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm00,0x0x0),NN),0x0x0_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 
 void Mjoin(Mjoin(Mjoin(NCmm0,NT),0x0x0),_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NT),0x0x0),_a1_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NT),0x0x0),_aX_b1)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NT),0x0x0),_a1_b1)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NT),0x0x0),_aX_b0)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,NT),0x0x0),_a1_b0)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm00,Mjoin(0x0x,KB)),NT),0x0x0_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm00,0x0x0),NT),0x0x0_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 
 void Mjoin(Mjoin(Mjoin(NCmm0,TN),0x0x0),_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TN),0x0x0),_a1_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TN),0x0x0),_aX_b1)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TN),0x0x0),_a1_b1)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TN),0x0x0),_aX_b0)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TN),0x0x0),_a1_b0)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm00,Mjoin(0x0x,KB)),TN),0x0x0_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm00,0x0x0),TN),0x0x0_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 
 void Mjoin(Mjoin(Mjoin(NCmm0,TT),0x0x0),_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TT),0x0x0),_a1_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TT),0x0x0),_aX_b1)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TT),0x0x0),_a1_b1)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TT),0x0x0),_aX_b0)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm0,TT),0x0x0),_a1_b0)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm00,Mjoin(0x0x,KB)),TT),0x0x0_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(Mjoin(Mjoin(NCmm00,0x0x0),TT),0x0x0_aX_bX)
-   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A, 
-    const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha, const TYPE *A,
+    const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 
 
@@ -191,13 +191,13 @@ void Mjoin(Mjoin(Mjoin(NCmm00,0x0x0),TT),0x0x0_aX_bX)
 #endif
 
 typedef void (*MMPTR)
-   (const int M, const int N, const int K, const TYPE alpha, 
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+   (const int M, const int N, const int K, const TYPE alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const TYPE beta, TYPE *C, const int ldc);
 int Mjoin(PATL,NCmmJIK_c)
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
     const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb, 
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc)
 /*
  * JIK loop-ordered matmul with no matrix copy
@@ -399,7 +399,7 @@ int Mjoin(PATL,NCmmJIK_c)
                r1mm(MB, NB, KB, ATL_rone, a, lda, b, ldb, ATL_rnone, cp, MB);
                i1mm(MB, NB, KB, ai1, a, lda, b+1, ldb, ATL_rone, cp+1, MB);
             }
-            if (kr) 
+            if (kr)
             {
                mmcu(MB, NB, kr, ar0, a+1, lda, b+1, ldb, ATL_rnone, cp, MB);
                mmcu(MB, NB, kr, ai0, a+1, lda, b, ldb, ATL_rone, cp+1, MB);
@@ -407,8 +407,9 @@ int Mjoin(PATL,NCmmJIK_c)
                mmcu(MB, NB, kr, ai1, a, lda, b+1, ldb, ATL_rone, cp+1, MB);
             }
          }
-         else if (kr) 
+         else if (kr)
          {
+            Mjoin(PATL,zero)(MB*NB, cp, 1); /* kill NaN/INF from before */
             mmcu(MB, NB, kr, ar0, a+1, lda, b+1, ldb, ATL_rzero, cp, MB);
             mmcu(MB, NB, kr, ai0, a+1, lda, b, ldb, ATL_rzero, cp+1, MB);
             mmcu(MB, NB, kr, ATL_rone, a, lda, b, ldb, ATL_rnone, cp, MB);
@@ -418,35 +419,36 @@ int Mjoin(PATL,NCmmJIK_c)
       }
    }
    if (mr && N != nr)
-      Mjoin(PATL,NCmmIJK)(TA, TB, mr, N-nr, K, alpha, A+Mb*(incAm+Kb*incAk), 
+      Mjoin(PATL,NCmmIJK)(TA, TB, mr, N-nr, K, alpha, A+Mb*(incAm+Kb*incAk),
                           lda, B, ldb, beta, C+Mb*(MB<<1), ldc);
    if (nr)
    {
       for (i=Mb; i; i--, a += incAm, b += incBm, c += incCm)
       {
+            Mjoin(PATL,zero)(MB*nr, cp, 1); /* kill NaN/INF from before */
          if (Kb)
          {
-            mm_fixedKcu(MB, nr, KB, ar0, a+1, lda, b+1, ldb, ATL_rzero, 
+            mm_fixedKcu(MB, nr, KB, ar0, a+1, lda, b+1, ldb, ATL_rzero,
                         cp, MB);
-            mm_fixedKcu(MB, nr, KB, ai0, a+1, lda, b, ldb, ATL_rzero, 
+            mm_fixedKcu(MB, nr, KB, ai0, a+1, lda, b, ldb, ATL_rzero,
                         cp+1, MB);
-            mm_fixedKcu(MB, nr, KB, ATL_rone, a, lda, b, ldb, ATL_rnone, 
+            mm_fixedKcu(MB, nr, KB, ATL_rone, a, lda, b, ldb, ATL_rnone,
                         cp, MB);
-            mm_fixedKcu(MB, nr, KB, ai1, a, lda, b+1, ldb, ATL_rone, 
+            mm_fixedKcu(MB, nr, KB, ai1, a, lda, b+1, ldb, ATL_rone,
                         cp+1, MB);
             a += incAk;  b += incBk;
             for (k=Kb-1; k; k--, a += incAk, b += incBk)
             {
-               mm_fixedKcu(MB, nr, KB, ar0, a+1, lda, b+1, ldb, ATL_rnone, 
+               mm_fixedKcu(MB, nr, KB, ar0, a+1, lda, b+1, ldb, ATL_rnone,
                            cp, MB);
-               mm_fixedKcu(MB, nr, KB, ai0, a+1, lda, b, ldb, ATL_rone, 
+               mm_fixedKcu(MB, nr, KB, ai0, a+1, lda, b, ldb, ATL_rone,
                            cp+1, MB);
-               mm_fixedKcu(MB, nr, KB, ATL_rone, a, lda, b, ldb, ATL_rnone, 
+               mm_fixedKcu(MB, nr, KB, ATL_rone, a, lda, b, ldb, ATL_rnone,
                            cp, MB);
-               mm_fixedKcu(MB, nr, KB, ai1, a, lda, b+1, ldb, ATL_rone, 
+               mm_fixedKcu(MB, nr, KB, ai1, a, lda, b+1, ldb, ATL_rone,
                            cp+1, MB);
             }
-            if (kr) 
+            if (kr)
             {
                mmcu(MB, nr, kr, ar0, a+1, lda, b+1, ldb, ATL_rnone, cp, MB);
                mmcu(MB, nr, kr, ai0, a+1, lda, b, ldb, ATL_rone, cp+1, MB);
@@ -454,7 +456,7 @@ int Mjoin(PATL,NCmmJIK_c)
                mmcu(MB, nr, kr, ai1, a, lda, b+1, ldb, ATL_rone, cp+1, MB);
             }
          }
-         else if (kr) 
+         else if (kr)
          {
             mmcu(MB, nr, kr, ar0, a+1, lda, b+1, ldb, ATL_rzero, cp, MB);
             mmcu(MB, nr, kr, ai0, a+1, lda, b, ldb, ATL_rzero, cp+1, MB);
@@ -468,29 +470,30 @@ int Mjoin(PATL,NCmmJIK_c)
          c = C + ((Mb*MB + ldc*Nb*NB)<<1);
          a = A + Mb*(incAm+Kb*incAk);
          b = B + Nb*( incBn+(Mb*(incBm+Kb*incBk)) );
+         Mjoin(PATL,zero)(MB*nr, cp, 1); /* kill NaN/INF from before */
          if (Kb)
          {
-            mm_fixedKcu(mr, nr, KB, ar0, a+1, lda, b+1, ldb, ATL_rzero, 
+            mm_fixedKcu(mr, nr, KB, ar0, a+1, lda, b+1, ldb, ATL_rzero,
                         cp, MB);
-            mm_fixedKcu(mr, nr, KB, ai0, a+1, lda, b, ldb, ATL_rzero, 
+            mm_fixedKcu(mr, nr, KB, ai0, a+1, lda, b, ldb, ATL_rzero,
                         cp+1, MB);
-            mm_fixedKcu(mr, nr, KB, ATL_rone, a, lda, b, ldb, ATL_rnone, 
+            mm_fixedKcu(mr, nr, KB, ATL_rone, a, lda, b, ldb, ATL_rnone,
                         cp, MB);
-            mm_fixedKcu(mr, nr, KB, ai1, a, lda, b+1, ldb, ATL_rone, 
+            mm_fixedKcu(mr, nr, KB, ai1, a, lda, b+1, ldb, ATL_rone,
                         cp+1, MB);
             a += incAk;  b += incBk;
             for (k=Kb-1; k; k--, a += incAk, b += incBk)
             {
-               mm_fixedKcu(mr, nr, KB, ar0, a+1, lda, b+1, ldb, ATL_rnone, 
+               mm_fixedKcu(mr, nr, KB, ar0, a+1, lda, b+1, ldb, ATL_rnone,
                            cp, MB);
-               mm_fixedKcu(mr, nr, KB, ai0, a+1, lda, b, ldb, ATL_rone, 
+               mm_fixedKcu(mr, nr, KB, ai0, a+1, lda, b, ldb, ATL_rone,
                            cp+1, MB);
-               mm_fixedKcu(mr, nr, KB, ATL_rone, a, lda, b, ldb, ATL_rnone, 
+               mm_fixedKcu(mr, nr, KB, ATL_rone, a, lda, b, ldb, ATL_rnone,
                            cp, MB);
-               mm_fixedKcu(mr, nr, KB, ai1, a, lda, b+1, ldb, ATL_rone, 
+               mm_fixedKcu(mr, nr, KB, ai1, a, lda, b+1, ldb, ATL_rone,
                            cp+1, MB);
             }
-            if (kr) 
+            if (kr)
             {
                mmcu(mr, nr, kr, ar0, a+1, lda, b+1, ldb, ATL_rnone, cp, MB);
                mmcu(mr, nr, kr, ai0, a+1, lda, b, ldb, ATL_rone, cp+1, MB);
@@ -498,7 +501,7 @@ int Mjoin(PATL,NCmmJIK_c)
                mmcu(mr, nr, kr, ai1, a, lda, b+1, ldb, ATL_rone, cp+1, MB);
             }
          }
-         else if (kr) 
+         else if (kr)
          {
             mmcu(mr, nr, kr, ar0, a+1, lda, b+1, ldb, ATL_rzero, cp, MB);
             mmcu(mr, nr, kr, ai0, a+1, lda, b, ldb, ATL_rzero, cp+1, MB);
