@@ -1,6 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.0Beta
- *                    (C) Copyright 1999 R. Clint Whaley                     
+ *             Automatically Tuned Linear Algebra Software v3.8.3
+ *                    (C) Copyright 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -10,21 +10,21 @@
  *   2. Redistributions in binary form must reproduce the above copyright
  *      notice, this list of conditions, and the following disclaimer in the
  *      documentation and/or other materials provided with the distribution.
- *   3. The name of the University, the ATLAS group, or the names of its 
- *      contributers may not be used to endorse or promote products derived
- *      from this software without specific written permission.
+ *   3. The name of the ATLAS group or the names of its contributers may
+ *      not be used to endorse or promote products derived from this
+ *      software without specific written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE. 
+ * POSSIBILITY OF SUCH DAMAGE.
  *
  */
 
@@ -45,7 +45,7 @@ static void gemvNle4(const int M, const int N, const TYPE *A, const int lda,
 #else
    #define Yget(y_, yp_, bet_) (y_) = (yp_)
 #endif
-static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
+static void gemvN32x4(const int M, const int N, const TYPE *A, const int lda,
                       const TYPE *x, const SCALAR beta0, TYPE *y)
 /*
  * rank-4 daxpy based NoTrans gemv
@@ -69,14 +69,14 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
       #ifdef BETA0
          y0 = y1 = y2 = y3 = y4 = y5 = y6 = y7 = ATL_rzero;
       #else
-         y0 = *y;   y1 = y[1]; y2 = y[2]; y3 = y[3]; 
-         y4 = y[4]; y5 = y[5]; y6 = y[6]; y7 = y[7]; 
+         y0 = *y;   y1 = y[1]; y2 = y[2]; y3 = y[3];
+         y4 = y[4]; y5 = y[5]; y6 = y[6]; y7 = y[7];
          #ifdef BETAX
             y0 *= beta; y1 *= beta; y2 *= beta; y3 *= beta;
             y4 *= beta; y5 *= beta; y6 *= beta; y7 *= beta;
          #endif
       #endif
-      y0 += x0 * *A0;   Yget(z0, y[8], beta);   
+      y0 += x0 * *A0;   Yget(z0, y[8], beta);
       y1 += x1 * A1[1];
       y2 += x2 * A2[2]; Yget(z1, y[9], beta);
       y3 += x3 * A3[3];
@@ -103,7 +103,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
       y6 += x0 * A0[6];
       y7 += x1 * A1[7];
 
-      y0 += x3 * *A3; 
+      y0 += x3 * *A3;
       y1 += x0 * A0[1];
       y2 += x1 * A1[2];
       y3 += x2 * A2[3];
@@ -113,7 +113,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
       y7 += x2 * A2[7];
 
       z0 += x0 * A0[8]; *y = y0;
-      z1 += x1 * A1[9]; 
+      z1 += x1 * A1[9];
       z2 += x2 * A2[10]; y[1] = y1;
       z3 += x3 * A3[11];
       z4 += x0 * A0[12]; y[2] = y2;
@@ -159,7 +159,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
             y5 += x1 * A1[5];
             y6 += x2 * A2[6]; y[11] = z3;
             y7 += x3 * A3[7];
-   
+
             y0 += x1 * *A1;   y[12] = z4;
             y1 += x2 * A2[1];
             y2 += x3 * A3[2]; y[13] = z5;
@@ -168,7 +168,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
             y5 += x2 * A2[5];
             y6 += x3 * A3[6]; y[15] = z7; y += 16;
             y7 += x0 * A0[7];
-   
+
             y0 += x2 * *A2;   Yget(z0, y[8], beta);
             y1 += x3 * A3[1];
             y2 += x0 * A0[2]; Yget(z1, y[9], beta);
@@ -177,7 +177,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
             y5 += x3 * A3[5];
             y6 += x0 * A0[6]; Yget(z3, y[11], beta);
             y7 += x1 * A1[7];
-   
+
             y0 += x3 * *A3;   Yget(z4, y[12], beta);
             y1 += x0 * A0[1];
             y2 += x1 * A1[2]; Yget(z5, y[13], beta);
@@ -186,7 +186,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
             y5 += x0 * A0[5];
             y6 += x1 * A1[6]; Yget(z7, y[15], beta);
             y7 += x2 * A2[7];
-   
+
             z0 += x0 * A0[8];  *y = y0;
             z1 += x1 * A1[9];
             z2 += x2 * A2[10]; y[1] = y1;
@@ -195,7 +195,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
             z5 += x1 * A1[13];
             z6 += x2 * A2[14]; y[3] = y3;
             z7 += x3 * A3[15];
-   
+
             z0 += x1 * A1[8]; y[4] = y4;
             z1 += x2 * A2[9];
             z2 += x3 * A3[10]; y[5] = y5;
@@ -204,7 +204,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
             z5 += x2 * A2[13];
             z6 += x3 * A3[14]; y[7] = y7;
             z7 += x0 * A0[15];
-   
+
             z0 += x2 * A2[8];  Yget(y0, y[16], beta);
             z1 += x3 * A3[ 9];
             z2 += x0 * A0[10]; Yget(y1, y[17], beta);
@@ -213,7 +213,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
             z5 += x3 * A3[13];
             z6 += x0 * A0[14]; Yget(y3, y[19], beta);
             z7 += x1 * A1[15];
-   
+
             z0 += x3 * A3[8];  Yget(y4, y[20], beta);
             z1 += x0 * A0[9];
             z2 += x1 * A1[10]; Yget(y5, y[21], beta);
@@ -242,7 +242,7 @@ static void gemvT32x4(const int M, const int N, const TYPE *A, const int lda,
       y5 += x2 * A2[5];
       y6 += x3 * A3[6]; y[15] = z7; y += 16;
       y7 += x0 * A0[7];
-   
+
       y0 += x2 * *A2;   Yget(z0, y[8], beta);
       y1 += x3 * A3[1];
       y2 += x0 * A0[2]; Yget(z1, y[9], beta);
@@ -323,14 +323,13 @@ static void gemv32x4(const int M, const int N, const TYPE *A, const int lda,
       {
          #ifdef BETA1
             for (j=(N>>2); j; j--, A += incA, X += 4)
-               gemvT32x4(M, 4, A, lda, X, ATL_rone, Y);
+               gemvN32x4(M, 4, A, lda, X, ATL_rone, Y);
             if ( (j = N-((N>>2)<<2)) ) gemvNle4(M, j, A, lda, X, ATL_rone, Y);
          #else
-            gemvT32x4(M, 4, A, lda, X, beta, Y);
+            gemvN32x4(M, 4, A, lda, X, beta, Y);
             if (N != 4)
                Mjoin(PATL,gemvN_a1_x1_b1_y1)
-                  (M, N-4, ATL_rone, A+incA, lda, X+4, 1, 
-                   ATL_rone, Y, 1);
+                  (M, N-4, ATL_rone, A+incA, lda, X+4, 1, ATL_rone, Y, 1);
          #endif
       }
       else gemvMlt8(M, N, A, lda, X, beta, Y);
@@ -369,7 +368,7 @@ static void gemvNle4(const int M, const int N, const TYPE *A, const int lda,
    {
    case 1:
       #if defined(BETA0)
-         Mjoin(PATL,move)(M, *X, A, 1, Y, 1);
+         Mjoin(PATL,cpsc)(M, *X, A, 1, Y, 1);
       #elif defined(BETAX)
          Mjoin(PATL,axpby)(M, *X, A, 1, beta, Y, 1);
       #else
@@ -377,7 +376,7 @@ static void gemvNle4(const int M, const int N, const TYPE *A, const int lda,
       #endif
       break;
    case 2:
-      x0 = *X; x1 = X[1]; 
+      x0 = *X; x1 = X[1];
       for (i=0; i != M; i++)
       #ifdef BETA0
          Y[i] = A0[i] * x0 + A1[i] * x1;
@@ -413,7 +412,8 @@ static void gemvNle4(const int M, const int N, const TYPE *A, const int lda,
          #endif
       }
       break;
-   default:;
+   default:
+      ATL_assert(!N);
    }
 }
 
@@ -425,7 +425,7 @@ static void gemv16x2(const int M, const int N, const TYPE *A, const int lda,
 {
    const int M16 = (M>>4)<<4, N2 = (N>>1)<<1, nr = N-N2;
    const int incA = lda << 1, incAm = 16 - N2*lda;
-   const TYPE *stX = X + N2 - 2, *x; 
+   const TYPE *stX = X + N2 - 2, *x;
    const TYPE *A0 = A, *A1 = A + lda;
    TYPE *stY = Y + M16;
    register TYPE x0, x1;
@@ -440,7 +440,7 @@ static void gemv16x2(const int M, const int N, const TYPE *A, const int lda,
          do
          {
             #ifdef BETA0
-               y0 = y1 = y2 = y3 = y4 = y5 = y6 = y7 = 
+               y0 = y1 = y2 = y3 = y4 = y5 = y6 = y7 =
                y8 = y9 = y10 = y11 = y12 = y13 = y14 = y15 = ATL_rzero;
             #elif defined BETAX
                x0 = beta;
@@ -623,4 +623,3 @@ void Mjoin(Mjoin(Mjoin(Mjoin(Mjoin(PATL,gemvN),NM),_x1),BNM),_y1)
 {
    gemv16x2(M, N, A, lda, X, beta, Y);
 }
-
